
import java.applet.Applet;
import java.applet.AudioClip;
import java.awt.event.ActionEvent;
import java.awt.image.BufferedImage;
import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.URL;
import javax.imageio.ImageIO;
import java.io.*;


/**
 * Clase que crea un componente que muestra la Leyenda de los gráficos
 * y además en caso de que el estanque 1 se vacíe se muestra una imagen de trompeta
 * intermitente junto con un sonido de alerta para informar de la situación.
 * @author Joaquín Moreira y Javier Nuñez
 */
public class Leyenda extends JComponent implements ActionListener{
    
    private boolean vacio=false,intermitencia;
    private Timer timer;
    private BufferedImage imagen;
    private long secuencia;
    private AudioClip sonido;
    private URL url;
        
    /**
     * Constructor inicializa los recursos de la imagen de la trompeta y el sonido
     * de la trompeta
     */
    public Leyenda(){
        try {
            url = getClass().getResource("trompeta.png");  //it works for applications and applets.
            imagen = ImageIO.read(url);
            url = getClass().getResource("trompeta.wav");
            sonido = Applet.newAudioClip(url);
        }
        catch(IOException e){}
        timer=new Timer(1000,this);
    }
    
    /**
     * Dibuja la leyenda de los gráficos y en caso de ser necesario
     * dibuja la imagen de la trompeta de alerta
     * @param g
     */
    @Override
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D) g;
        g2.setColor(Color.BLUE);
        g2.drawLine(20, 20, 100, 20);
        g2.drawString("Nivel Tanque", 110, 20);

        g2.setColor(Color.RED);
        g2.drawLine(20, 40, 100, 40);
        g2.drawString("Apertura Valvula", 110, 40);

        g2.setColor(Color.ORANGE);
        g2.drawLine(20, 60, 100, 60);
        g2.drawString("Altura Referencia", 110, 60);
        
        //Si esta vacío el tanque y si corresponde la intermitencia
        //Intermitencia es un numero de secuencia que controla si se debe
        //mostrar o ocultar la imagen
        if(vacio & intermitencia){
            g2.drawImage(imagen, null, 70, 70);
            
            intermitencia=false;
        }
    }
    
    /**
     * Activa la alerta de la trompeta
     */
    public void showAlert(){
        vacio=true;
        intermitencia=true;
        secuencia=0;
        timer.start();
        sonido.play();
        sonido.loop();
    }
    /**
     * Desactiva la alerta de la trompeta
     */
    public void endAlert(){
        vacio=false;
        timer.stop();
        sonido.stop();
        repaint();
    }

    /**
     * Método que responde al timer y se activa por este.
     * Se encarga de asegurar la intermitencia de la imagen de la trompeta
     * @param arg0
     */
    public void actionPerformed(ActionEvent arg0) {
        if(vacio){
            //Mostrar imagen
            if(secuencia%2==0){
                intermitencia=true;
                repaint();
            }
            //Ocultar imagen
            else{
                intermitencia=false;
                repaint();
            }
            secuencia++;
        }
    }
}