
/**
@version 1.0 06-05-2008
@author Agustin J. Gonzalez
 */
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;

/**
A panel with two tanks and a control system. A simulation to practice 
with applets.
 */
public class PlantPanel extends JPanel {

    
    public PlantPanel() {        
        menuPanel = new Box(BoxLayout.Y_AXIS);
        
        //grafPanel almacena los componentes de los gráficos y de la leyenda
        grafPanel=new Box(BoxLayout.Y_AXIS);
        grafPanel.setAlignmentY(JComponent.CENTER_ALIGNMENT);
        grafPanel.setPreferredSize(new Dimension(300,600));
        grafPanel.setBorder(BorderFactory.createTitledBorder("Gráficos Tiempo Real"));
                
        view = new PlantView();
        view.setPreferredSize(new Dimension(260,500));

        dt = 0.05F; // 50 ms is the delta t of the simulation. 
        TimePace timer = new TimePace(this);
        menuPanel.add(timer.getGUI());
        menuPanel.add(Box.createRigidArea(new Dimension(10, 0)));

        tank1 = new Tank(new Point(110, 140), view, 1);
        menuPanel.add(tank1.getGUI());
        menuPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        view.addEquipment(tank1);

        tank2 = new Tank(new Point(150, 250), view, 2);
        menuPanel.add(tank2.getGUI());
        menuPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        view.addEquipment(tank2);

        tank1.setReceiver(tank2);

        Controller control = new Controller(new Point(2, 360), tank2);
        view.addEquipment(control);
        menuPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        menuPanel.add(control.getGUI());
        
        //Inicializa grafico de tanque 1
        grafico1=new Grafico(new Point(20,10),tank1);
        grafico1.enableNivel();
        grafico1.enableValvula();
        grafico1.start();
        
        //Inicializa grafico de tanque2
        grafico2=new Grafico(new Point(20,10), tank2,control);
        grafico2.enableNivel();
        grafico2.enableValvula();
        grafico2.enableRef();
        grafico2.start();
        
        leyenda=new Leyenda();
        
        grafPanel.add(grafico1);
        grafPanel.add(grafico2);
        grafPanel.add(leyenda);
        
        add(view);
        add(grafPanel);
        add(menuPanel);
        
        timer.start();
    }

    /**
     * Controla las excepciones en caso de que el estanque este vacío o que haya
     * dejado de estar vacío.
     */
    public void simulationTick() {
        try{
            view.timeTick(dt);
        }
        catch(Exception e){
            if(e.getMessage().equals("1")){
                //Estanque Vacío
                leyenda.showAlert();
            }
            else{
                //Estanque deja de estar Vacío
                leyenda.endAlert();
            }
        }      
        view.repaint();
    }
    
    private JComponent menuPanel;
    private PlantView view;
    private JComponent grafPanel;
    private Grafico grafico1;
    private Grafico grafico2;
    private Leyenda leyenda;
    private float dt;
    private static final long serialVersionUID = 2L;
    private Tank tank1,tank2;
}
